/*
*
*  $Id: comandopacs.cpp $
*  Ginkgo CADx Project
*
*  Copyright 2008-12 MetaEmotion S.L. All rights reserved.
*  http://ginkgo-cadx.com
*
*  This file is licensed under LGPL v3 license.
*  See License.txt for details
*
*/
//#define _GINKGO_TRACE

#include <sstream>
#include <map>

#include <api/api.h>
#include <api/globals.h>
#include <api/ientorno.h>

#include <api/controllers/icontroladorlog.h>
#include <api/controllers/icommandscontroller.h>
#include <api/controllers/ieventscontroller.h>
#include <api/controllers/icontroladorpermisos.h>

#include <eventos/mensajes.h>

#include <main/controllers/dcmtk/dicomservers.h>

#include "comandopacs.h"
#include "comandoincluirhistorial.h"



#define IDC_PACS_BUSCAR                  61
#define IDC_PACS_DESCARGAR               62
#define IDC_PACS_ABORTAR_TODOS           63
#define IDC_PACS_SUBIR			         64

#define ID_REFRESCO_PROGRESO 1

#define PARALLEL

// Singleton de persistencia
namespace GADAPI
{

	ComandoPACS::ComandoPACS(ComandoPACSParams* pParams) : IComando(pParams)
	{
		m_pPACSParams = pParams;
		switch (m_pPACSParams->m_TipoAccion) {
			//no es posible hacer dos comandos de estos en paralelo dcmtk no es multithread
			case GADAPI::PACS::TAC_Buscar :
				SetId(IDC_PACS_BUSCAR);
#if !defined(PARALLEL)
				EsperaA(IDC_PACS_BUSCAR);
				EsperaA(IDC_PACS_DESCARGAR);
				EsperaA(IDC_PACS_SUBIR);
#endif
				AbortaSi(IDC_PACS_ABORTAR_TODOS);
				break;
			case GADAPI::PACS::TAC_Descargar :
				SetId(IDC_PACS_DESCARGAR);
				EsperaA(IDC_PACS_DESCARGAR);
#if !defined(PARALLEL)
				EsperaA(IDC_PACS_BUSCAR);
				EsperaA(IDC_PACS_DESCARGAR);
				EsperaA(IDC_PACS_SUBIR);
#endif
				AbortaSi(IDC_PACS_ABORTAR_TODOS);
				break;
		}
	}

	void ComandoPACS::Execute()
	{
		if(m_pPACSParams->m_TipoAccion == GADAPI::PACS::TAC_Buscar)
		{
			Search();
		} else if(m_pPACSParams->m_TipoAccion == GADAPI::PACS::TAC_Descargar){
			Download();
		}
	}

	void ComandoPACS::Update()
	{

		if (EstaAbortado()) {
		    return;
		}
		
		if(m_pPACSParams->m_TipoAccion == GADAPI::PACS::TAC_Buscar) {
			if(m_pPACSParams->m_pNotificador != NULL){
				if(!m_pPACSParams->m_error.empty()){
					LOG_ERROR("C-FIND", "Error searching on PACS:\n"  << m_pPACSParams->m_error);
					if (m_pPACSParams->m_informar) {
						GNC::GCS::IEventsController::Instance()->ProcesarEvento(new GNC::GCS::Events::EventoMensajes(NULL, _Std("Failed to perform search: ") + "\n" + m_pPACSParams->m_error, GNC::GCS::Events::EventoMensajes::PopUpMessage,false, GNC::GCS::Events::EventoMensajes::Error));
					}
					m_pPACSParams->m_informar = false;
					m_pPACSParams->m_pNotificador->PACSCargarListado(m_pPACSParams->m_pModelo);
				} else {
					m_pPACSParams->m_pNotificador->PACSCargarListado(m_pPACSParams->m_pModelo);
				}
			} else {
				// There is no notifier specified
			}
		}
		else if(m_pPACSParams->m_TipoAccion == GADAPI::PACS::TAC_Descargar) {
			if(!m_pPACSParams->m_error.empty()){
				LOG_ERROR("C-MOVE/C-GET", "Error Downloading study: " << m_pPACSParams->m_error);
				if (m_pPACSParams->m_informar) {
					GNC::GCS::IEventsController::Instance()->ProcesarEvento(new GNC::GCS::Events::EventoMensajes(NULL, _Std("Error downloading study: ") + "\n" + m_pPACSParams->m_error, GNC::GCS::Events::EventoMensajes::PopUpMessage,false, GNC::GCS::Events::EventoMensajes::Error));
				}
				m_pPACSParams->m_informar = false;
				return;
			}
			else {

				std::list<std::string> rutas;
				std::string rutaImagen;
				if (m_pPACSParams->m_pModelo.IsValid()) {
					//listamos las imagenes que nos hemos bajado
					for (IModeloDicom::ListaPacientesType::const_iterator it = m_pPACSParams->m_pModelo->ListaPacientes().begin(); it != m_pPACSParams->m_pModelo->ListaPacientes().end(); ++it) {
						const IModeloPaciente& p = *it;
						for (IModeloPaciente::ListaEstudiosType::const_iterator it2 = p.ListaEstudios().begin(); it2 != p.ListaEstudios().end(); ++it2) {
							const IModeloEstudio& e = *it2;
							for (IModeloEstudio::ListaSeriesType::const_iterator it3 = e.ListaSeries().begin(); it3 != e.ListaSeries().end(); ++it3) {
								const IModeloSerie& s = *it3;
								for (IModeloSerie::ListaImagenesType::const_iterator it4 = s.ListaImagenes().begin(); it4 != s.ListaImagenes().end(); ++it4) {
									const IModeloImagen& im = *it4;
									if(GNC::GCS::IEntorno::Instance()->GetPACSController()->GetRutaImagenTemp(p.GetUID(),e.GetUID(),s.GetUID(),im.GetUID(), rutaImagen)){
										rutas.push_back(rutaImagen);
									}
								}
							}
						}
					}
				}

				//si ha descargado algo...
				if(rutas.empty()){
					return;
				}

				//se lanza un evento para incluir las imagenes en el historial
				GADAPI::ComandoIncluirHistorial::ComandoIncluirHistorialParams* pParams = new GADAPI::ComandoIncluirHistorial::ComandoIncluirHistorialParams(rutas, m_pPACSParams->m_informar, GNC::GCS::IHistoryController::TAA_MOVE);
				pParams->m_informar = m_pPACSParams->m_informar;
				GADAPI::ComandoIncluirHistorial::ComandoIncluirHistorial * pCmd = new GADAPI::ComandoIncluirHistorial::ComandoIncluirHistorial(pParams);
				GNC::GCS::IControladorComandos::Instance()->ProcessAsync(_Std("Including files..."), pCmd, NULL);
			}
		}
	}

	void ComandoPACS::Search()
	{
		std::string tarea=_Std("Querying PACS ...");
		if (!NotificarProgreso(0.0f,tarea)) {
		    return;
		}
		GIL::DICOM::IPACSController* pCI = NULL;
		try {

			pCI = GNC::GCS::IEntorno::Instance()->GetPACSController();

			pCI->GetConnection(this);

			if (pCI == NULL) {
				throw GIL::DICOM::PACSException(_Std("Error accessing the controller subsystem integration: (GIL:: DICOM)."));
			}
			if (!m_pPACSParams->m_pModelo.IsValid()) {
				throw GIL::DICOM::PACSException(_Std("Subsystem integration error: uninitialized result data container."));
			}

			std::string mensaje = _Std("Starting Search ...");
			if ( !NotificarProgreso(0.0f, mensaje) ) {
			    return;
			}

			switch (m_pPACSParams->m_Ambito) {
				case ComandoPACSParams::TA_Paciente:
					//std::cout << "patient" << std::endl;
					break;
				case ComandoPACSParams::TA_Estudio:
					pCI->BuscarEstudio(
						this,
						m_pPACSParams->m_serverSeleccionado, // Server ID
						m_pPACSParams->m_pacienteID, // ID Paciente
						m_pPACSParams->m_pacienteNombre, // Nombre Paciente
						m_pPACSParams->m_studyUID,
						m_pPACSParams->m_accessionNumber, // Accesion Number
						m_pPACSParams->m_estudioModalidad, // Modalidad
						m_pPACSParams->m_estudioFechaDesde, // Fecha Desde
						m_pPACSParams->m_estudioFechaHasta, // Fecha Hasta
						m_pPACSParams->m_studyTimeFrom,  // time from
						m_pPACSParams->m_studyTimeTo,		// time To
						"", // Description
						"", // Nombre de la estación
						m_pPACSParams->m_pModelo,
						this);
					break;
				case ComandoPACSParams::TA_Serie:
					{
						const IModeloEstudio* estudio = NULL;
						m_pPACSParams->m_pModelo->BuscarEstudio(m_pPACSParams->m_studyUID,&estudio);
						if (estudio == NULL) {
							pCI->BuscarEstudio(
								this,
								m_pPACSParams->m_serverSeleccionado, // Server ID
								"", // ID Paciente
								"", // Nombre Paciente
								m_pPACSParams->m_studyUID,
								"", // Accesion Number
								"", // Modalidad
								"", // Fecha Desde
								"", // Fecha Hasta
								"", // time from
								"", // time to
								"", // Descripcion del Estudio
								"", // Nombre de la estación
								m_pPACSParams->m_pModelo,this
							);
						}
						GNC::GCS::IEntorno::Instance()->GetPACSController()->BuscarSeries(
							this,
							m_pPACSParams->m_serverSeleccionado, // Server ID
							m_pPACSParams->m_studyUID,
							std::string(),
							m_pPACSParams->m_pModelo,this
						);
					}
					break;
				default:
					break;
			}
			GTRACE(m_pPACSParams->m_pModelo);
		} catch (GinkgoNoServerFoundException& ) {
			m_pPACSParams->m_error= _Std("Server ID not found. ID = ") + m_pPACSParams->m_serverSeleccionado;
			m_pPACSParams->m_pModelo->Error = true;
		} catch (GIL::DICOM::PACSException& ex) {
			m_pPACSParams->m_error= _Std("Query error with PACS Id ") + m_pPACSParams->m_serverSeleccionado + "\n" + (const std::string)ex;
			m_pPACSParams->m_pModelo->Error = true;
		} catch (GIL::DICOM::ModelException& ex) {
			m_pPACSParams->m_error= _Std("Query error with PACS Id ") + m_pPACSParams->m_serverSeleccionado + "\n" + (const std::string)ex;
			m_pPACSParams->m_pModelo->Error = true;
		} catch (std::exception& ex) {
			m_pPACSParams->m_error= _Std("Query error with PACS Id ") + m_pPACSParams->m_serverSeleccionado + "\n" + ex.what();
			m_pPACSParams->m_pModelo->Error = true;
		} catch ( ... ) {
			m_pPACSParams->m_error= _Std("Query error with PACS Id ") + m_pPACSParams->m_serverSeleccionado + "\n" + _Std("Internal error");
			m_pPACSParams->m_pModelo->Error = true;
		}
		if (pCI) {
			pCI->ReleaseConnection(this);
		}
		if (!NotificarProgreso(1.0f,tarea)) {
		    return;
		}
	}

	void ComandoPACS::Download()
	{
		std::string tarea= _Std("Downloading files ...");
		if (!NotificarProgreso(0.0f,tarea)) {
		    return;
		}

		GIL::DICOM::IPACSController* pCI = NULL;

		try {
			pCI = GNC::GCS::IEntorno::Instance()->GetPACSController();
			pCI->GetConnection(this);
			pCI->QueryRetrieve(this, m_pPACSParams->m_serverSeleccionado, m_pPACSParams->m_pModelo, m_pPACSParams->m_base, this);
		}
		catch (GinkgoNoServerFoundException& ) {
			m_pPACSParams->m_error= _Std("Server ID not found. ID = ") + m_pPACSParams->m_serverSeleccionado;
			m_pPACSParams->m_pModelo->Error = true;
		}
		catch (GIL::DICOM::PACSException& ex) {
			m_pPACSParams->m_error =  (const std::string)ex;
			m_pPACSParams->m_pModelo->Error = true;
		}
		catch (std::exception& ex) {
			m_pPACSParams->m_error= ex.what();
			m_pPACSParams->m_pModelo->Error = true;
		}
		catch (...)
		{
			m_pPACSParams->m_error= _Std("Internal Error");
			m_pPACSParams->m_pModelo->Error = true;
		}
		if (pCI) {
			pCI->ReleaseConnection(this);
		}
		GTRACE(m_pPACSParams->m_pModelo);
		NotificarProgreso(1.0f,tarea);
	}

	bool ComandoPACS::NotificarProgreso(float progresoNormalizado,const std::string &texto) {
		if (EstaAbortado())
		{
			return false;
		}
		return IComando::NotificarProgreso(progresoNormalizado, texto);
	}

	void ComandoPACS::LiberarRecursos()
	{
	}

}
